@@ -28,8 +28,6 @@ Module attributes:
 """
 
 import os
-import ctypes
-import ctypes.util
 
 from PyQt5.QtGui import QFont
 from PyQt5.QtWebEngineWidgets import (QWebEngineSettings, QWebEngineProfile,
@@ -231,11 +229,6 @@ def init(args):
     if args.enable_webengine_inspector:
         os.environ['QTWEBENGINE_REMOTE_DEBUGGING'] = str(utils.random_port())
 
-    # WORKAROUND for
-    # https://bugs.launchpad.net/ubuntu/+source/python-qt4/+bug/941826
-    if utils.is_linux:
-        ctypes.CDLL(ctypes.util.find_library("GL"), mode=ctypes.RTLD_GLOBAL)
-
     _init_profiles()
 
     # We need to do this here as a WORKAROUND for
