@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2016 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -19,30 +19,36 @@
 
 """The main browser widget for QtWebEngine."""
 
-import os
 import functools
 
 from PyQt5.QtCore import pyqtSignal, pyqtSlot, QUrl, PYQT_VERSION
-# pylint: disable=no-name-in-module,import-error,useless-suppression
+from PyQt5.QtGui import QPalette
 from PyQt5.QtWebEngineWidgets import QWebEngineView, QWebEnginePage
-# pylint: enable=no-name-in-module,import-error,useless-suppression
 
 from qutebrowser.browser import shared
-from qutebrowser.browser.webengine import certificateerror
+from qutebrowser.browser.webengine import certificateerror, webenginesettings
 from qutebrowser.config import config
-from qutebrowser.utils import (log, debug, usertypes, qtutils, jinja, urlutils,
-                               message)
+from qutebrowser.utils import log, debug, usertypes, jinja, urlutils, message
 
 
 class WebEngineView(QWebEngineView):
 
     """Custom QWebEngineView subclass with qutebrowser-specific features."""
 
-    def __init__(self, tabdata, win_id, parent=None):
+    def __init__(self, *, tabdata, win_id, private, parent=None):
         super().__init__(parent)
         self._win_id = win_id
         self._tabdata = tabdata
-        self.setPage(WebEnginePage(parent=self))
+
+        theme_color = self.style().standardPalette().color(QPalette.Base)
+        if private:
+            profile = webenginesettings.private_profile
+            assert profile.isOffTheRecord()
+        else:
+            profile = webenginesettings.default_profile
+        page = WebEnginePage(theme_color=theme_color, profile=profile,
+                             parent=self)
+        self.setPage(page)
 
     def shutdown(self):
         self.page().shutdown()
@@ -67,23 +73,16 @@ class WebEngineView(QWebEngineView):
                          A window without decoration.
                      QWebEnginePage::WebBrowserBackgroundTab:
                          A web browser tab without hiding the current visible
-                         WebEngineView. (Added in Qt 5.7)
+                         WebEngineView.
 
         Return:
             The new QWebEngineView object.
         """
         debug_type = debug.qenum_key(QWebEnginePage, wintype)
-        background_tabs = config.get('tabs', 'background-tabs')
+        background = config.val.tabs.background
 
-        log.webview.debug("createWindow with type {}, background_tabs "
-                          "{}".format(debug_type, background_tabs))
-
-        try:
-            background_tab_wintype = QWebEnginePage.WebBrowserBackgroundTab
-        except AttributeError:
-            # This is unavailable with an older PyQt, but we still might get
-            # this with a newer Qt...
-            background_tab_wintype = 0x0003
+        log.webview.debug("createWindow with type {}, background {}".format(
+            debug_type, background))
 
         if wintype == QWebEnginePage.WebBrowserWindow:
             # Shift-Alt-Click
@@ -95,13 +94,13 @@ class WebEngineView(QWebEngineView):
         elif wintype == QWebEnginePage.WebBrowserTab:
             # Middle-click / Ctrl-Click with Shift
             # FIXME:qtwebengine this also affects target=_blank links...
-            if background_tabs:
+            if background:
                 target = usertypes.ClickTarget.tab
             else:
                 target = usertypes.ClickTarget.tab_bg
-        elif wintype == background_tab_wintype:
+        elif wintype == QWebEnginePage.WebBrowserBackgroundTab:
             # Middle-click / Ctrl-Click
-            if background_tabs:
+            if background:
                 target = usertypes.ClickTarget.tab_bg
             else:
                 target = usertypes.ClickTarget.tab
@@ -109,15 +108,6 @@ class WebEngineView(QWebEngineView):
             raise ValueError("Invalid wintype {}".format(debug_type))
 
         tab = shared.get_tab(self._win_id, target)
-
-        # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-54419
-        vercheck = qtutils.version_check
-        qtbug54419_fixed = ((vercheck('5.6.2') and not vercheck('5.7.0')) or
-                            qtutils.version_check('5.7.1') or
-                            os.environ.get('QUTE_QTBUG54419_PATCHED', ''))
-        if not qtbug54419_fixed:
-            tab.needs_qtbug54419_workaround = True
-
         return tab._widget  # pylint: disable=protected-access
 
 
@@ -127,6 +117,7 @@ class WebEnginePage(QWebEnginePage):
 
     Attributes:
         _is_shutting_down: Whether the page is currently shutting down.
+        _theme_color: The theme background color.
 
     Signals:
         certificate_error: Emitted on certificate errors.
@@ -136,21 +127,30 @@ class WebEnginePage(QWebEnginePage):
     certificate_error = pyqtSignal()
     shutting_down = pyqtSignal()
 
-    def __init__(self, parent=None):
-        super().__init__(parent)
+    def __init__(self, *, theme_color, profile, parent=None):
+        super().__init__(profile, parent)
         self._is_shutting_down = False
         self.featurePermissionRequested.connect(
             self._on_feature_permission_requested)
+        self._theme_color = theme_color
+        self._set_bg_color()
+        config.instance.changed.connect(self._set_bg_color)
+
+    @config.change_filter('colors.webpage.bg')
+    def _set_bg_color(self):
+        col = config.val.colors.webpage.bg
+        if col is None:
+            col = self._theme_color
+        self.setBackgroundColor(col)
 
     @pyqtSlot(QUrl, 'QWebEnginePage::Feature')
     def _on_feature_permission_requested(self, url, feature):
         """Ask the user for approval for geolocation/media/etc.."""
         options = {
-            QWebEnginePage.Geolocation: ('content', 'geolocation'),
-            QWebEnginePage.MediaAudioCapture: ('content', 'media-capture'),
-            QWebEnginePage.MediaVideoCapture: ('content', 'media-capture'),
-            QWebEnginePage.MediaAudioVideoCapture:
-                ('content', 'media-capture'),
+            QWebEnginePage.Geolocation: 'content.geolocation',
+            QWebEnginePage.MediaAudioCapture: 'content.media_capture',
+            QWebEnginePage.MediaVideoCapture: 'content.media_capture',
+            QWebEnginePage.MediaAudioVideoCapture: 'content.media_capture',
         }
         messages = {
             QWebEnginePage.Geolocation: 'access your location',
@@ -212,7 +212,7 @@ class WebEnginePage(QWebEnginePage):
         url_string = url.toDisplayString()
         error_page = jinja.render(
             'error.html', title="Error loading page: {}".format(url_string),
-            url=url_string, error=str(error), icon='')
+            url=url_string, error=str(error))
 
         if error.is_overridable():
             ignore = shared.ignore_certificate_errors(
@@ -274,19 +274,12 @@ class WebEnginePage(QWebEnginePage):
 
     def javaScriptConsoleMessage(self, level, msg, line, source):
         """Log javascript messages to qutebrowser's log."""
-        # FIXME:qtwebengine maybe unify this in the tab api somehow?
-        setting = config.get('general', 'log-javascript-console')
-        if setting == 'none':
-            return
-
-        level_to_logger = {
-            QWebEnginePage.InfoMessageLevel: log.js.info,
-            QWebEnginePage.WarningMessageLevel: log.js.warning,
-            QWebEnginePage.ErrorMessageLevel: log.js.error,
+        level_map = {
+            QWebEnginePage.InfoMessageLevel: usertypes.JsLogLevel.info,
+            QWebEnginePage.WarningMessageLevel: usertypes.JsLogLevel.warning,
+            QWebEnginePage.ErrorMessageLevel: usertypes.JsLogLevel.error,
         }
-        logstring = "[{}:{}] {}".format(source, line, msg)
-        logger = level_to_logger[level]
-        logger(logstring)
+        shared.javascript_log_message(level_map[level], source, line, msg)
 
     def acceptNavigationRequest(self,
                                 url: QUrl,
