@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2016 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -23,13 +23,12 @@ Defines a CompletionView which uses CompletionFiterModel and CompletionModel
 subclasses to provide completions.
 """
 
-from PyQt5.QtWidgets import QStyle, QTreeView, QSizePolicy
+from PyQt5.QtWidgets import QStyle, QTreeView, QSizePolicy, QStyleFactory
 from PyQt5.QtCore import pyqtSlot, pyqtSignal, Qt, QItemSelectionModel, QSize
 
-from qutebrowser.config import config, style
+from qutebrowser.config import config
 from qutebrowser.completion import completiondelegate
-from qutebrowser.completion.models import base
-from qutebrowser.utils import utils, usertypes, objreg
+from qutebrowser.utils import utils, usertypes, debug, log
 from qutebrowser.commands import cmdexc, cmdutils
 
 
@@ -41,6 +40,7 @@ class CompletionView(QTreeView):
     headers, and children show as flat list.
 
     Attributes:
+        pattern: Current filter pattern, used for highlighting.
         _win_id: The ID of the window this CompletionView is associated with.
         _height: The height to use for the CompletionView.
         _height_perc: Either None or a percentage if height should be relative.
@@ -57,27 +57,27 @@ class CompletionView(QTreeView):
     # don't define that in this stylesheet.
     STYLESHEET = """
         QTreeView {
-            font: {{ font['completion'] }};
-            background-color: {{ color['completion.bg'] }};
-            alternate-background-color: {{ color['completion.alternate-bg'] }};
+            font: {{ conf.fonts.completion.entry }};
+            background-color: {{ conf.colors.completion.even.bg }};
+            alternate-background-color: {{ conf.colors.completion.odd.bg }};
             outline: 0;
             border: 0px;
         }
 
         QTreeView::item:disabled {
-            background-color: {{ color['completion.category.bg'] }};
+            background-color: {{ conf.colors.completion.category.bg }};
             border-top: 1px solid
-                {{ color['completion.category.border.top'] }};
+                {{ conf.colors.completion.category.border.top }};
             border-bottom: 1px solid
-                {{ color['completion.category.border.bottom'] }};
+                {{ conf.colors.completion.category.border.bottom }};
         }
 
         QTreeView::item:selected, QTreeView::item:selected:hover {
             border-top: 1px solid
-                {{ color['completion.item.selected.border.top'] }};
+                {{ conf.colors.completion.item.selected.border.top }};
             border-bottom: 1px solid
-                {{ color['completion.item.selected.border.bottom'] }};
-            background-color: {{ color['completion.item.selected.bg'] }};
+                {{ conf.colors.completion.item.selected.border.bottom }};
+            background-color: {{ conf.colors.completion.item.selected.bg }};
         }
 
         QTreeView:item::hover {
@@ -85,14 +85,14 @@ class CompletionView(QTreeView):
         }
 
         QTreeView QScrollBar {
-            width: {{ config.get('completion', 'scrollbar-width') }}px;
-            background: {{ color['completion.scrollbar.bg'] }};
+            width: {{ conf.completion.scrollbar.width }}px;
+            background: {{ conf.colors.completion.scrollbar.bg }};
         }
 
         QTreeView QScrollBar::handle {
-            background: {{ color['completion.scrollbar.fg'] }};
-            border: {{ config.get('completion', 'scrollbar-padding') }}px solid
-                    {{ color['completion.scrollbar.bg'] }};
+            background: {{ conf.colors.completion.scrollbar.fg }};
+            border: {{ conf.completion.scrollbar.padding }}px solid
+                    {{ conf.colors.completion.scrollbar.bg }};
             min-height: 10px;
         }
 
@@ -107,23 +107,23 @@ class CompletionView(QTreeView):
 
     def __init__(self, win_id, parent=None):
         super().__init__(parent)
+        self.pattern = ''
         self._win_id = win_id
-        # FIXME handle new aliases.
-        # objreg.get('config').changed.connect(self.init_command_completion)
-        objreg.get('config').changed.connect(self._on_config_changed)
+        config.instance.changed.connect(self._on_config_changed)
 
-        self._column_widths = base.BaseCompletionModel.COLUMN_WIDTHS
         self._active = False
 
         self._delegate = completiondelegate.CompletionItemDelegate(self)
         self.setItemDelegate(self._delegate)
-        style.set_register_stylesheet(self)
+        self.setStyle(QStyleFactory.create('Fusion'))
+        config.set_register_stylesheet(self)
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
         self.setHeaderHidden(True)
         self.setAlternatingRowColors(True)
         self.setIndentation(0)
         self.setItemsExpandable(False)
         self.setExpandsOnDoubleClick(False)
+        self.setAnimated(False)
         self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         # WORKAROUND
         # This is a workaround for weird race conditions with invalid
@@ -139,17 +139,18 @@ class CompletionView(QTreeView):
     def __repr__(self):
         return utils.get_repr(self)
 
-    @pyqtSlot(str, str)
-    def _on_config_changed(self, section, option):
-        if section != 'completion':
-            return
-        if option in ['height', 'shrink']:
+    @pyqtSlot(str)
+    def _on_config_changed(self, option):
+        if option in ['completion.height', 'completion.shrink']:
             self.update_geometry.emit()
 
     def _resize_columns(self):
         """Resize the completion columns based on column_widths."""
+        if self.model() is None:
+            return
         width = self.size().width()
-        pixel_widths = [(width * perc // 100) for perc in self._column_widths]
+        column_widths = self.model().column_widths
+        pixel_widths = [(width * perc // 100) for perc in column_widths]
 
         if self.verticalScrollBar().isVisible():
             delta = self.style().pixelMetric(QStyle.PM_ScrollBarExtent) + 5
@@ -252,59 +253,66 @@ class CompletionView(QTreeView):
         selmodel.setCurrentIndex(
             idx, QItemSelectionModel.ClearAndSelect | QItemSelectionModel.Rows)
 
+        # if the last item is focused, try to fetch more
+        if idx.row() == self.model().rowCount(idx.parent()) - 1:
+            self.expandAll()
+
         count = self.model().count()
         if count == 0:
             self.hide()
-        elif count == 1 and config.get('completion', 'quick-complete'):
+        elif count == 1 and config.val.completion.quick:
             self.hide()
-        elif config.get('completion', 'show') == 'auto':
+        elif config.val.completion.show == 'auto':
             self.show()
 
-    def set_model(self, model, pattern=None):
+    def set_model(self, model):
         """Switch completion to a new model.
 
         Called from on_update_completion().
 
         Args:
             model: The model to use.
-            pattern: The filter pattern to set (what the user entered).
         """
+        if self.model() is not None and model is not self.model():
+            self.model().deleteLater()
+            self.selectionModel().deleteLater()
+
+        self.setModel(model)
+
         if model is None:
             self._active = False
             self.hide()
             return
 
-        old_model = self.model()
-        if model is not old_model:
-            sel_model = self.selectionModel()
+        model.setParent(self)
+        self._active = True
+        self._maybe_show()
 
-            self.setModel(model)
-            self._active = True
+        self._resize_columns()
+        for i in range(model.rowCount()):
+            self.expand(model.index(i, 0))
 
-            if sel_model is not None:
-                sel_model.deleteLater()
-            if old_model is not None:
-                old_model.deleteLater()
+    def set_pattern(self, pattern):
+        """Set the pattern on the underlying model."""
+        if not self.model():
+            return
+        self.pattern = pattern
+        with debug.log_time(log.completion, 'Set pattern {}'.format(pattern)):
+            self.model().set_pattern(pattern)
+            self.selectionModel().clear()
+            self._maybe_update_geometry()
+            self._maybe_show()
 
-        if (config.get('completion', 'show') == 'always' and
-                model.count() > 0):
+    def _maybe_show(self):
+        if (config.val.completion.show == 'always' and
+                self.model().count() > 0):
             self.show()
         else:
             self.hide()
 
-        for i in range(model.rowCount()):
-            self.expand(model.index(i, 0))
-
-        if pattern is not None:
-            model.set_pattern(pattern)
-
-        self._column_widths = model.srcmodel.COLUMN_WIDTHS
-        self._resize_columns()
-        self._maybe_update_geometry()
-
     def _maybe_update_geometry(self):
         """Emit the update_geometry signal if the config says so."""
-        if config.get('completion', 'shrink'):
+        if config.val.completion.shrink:
             self.update_geometry.emit()
 
     @pyqtSlot()
@@ -319,14 +327,14 @@ class CompletionView(QTreeView):
     def sizeHint(self):
         """Get the completion size according to the config."""
         # Get the configured height/percentage.
-        confheight = str(config.get('completion', 'height'))
+        confheight = str(config.val.completion.height)
         if confheight.endswith('%'):
             perc = int(confheight.rstrip('%'))
             height = self.window().height() * perc / 100
         else:
             height = int(confheight)
         # Shrink to content size if needed and shrinking is enabled
-        if config.get('completion', 'shrink'):
+        if config.val.completion.shrink:
             contents_height = (
                 self.viewportSizeHint().height() +
                 self.horizontalScrollBar().sizeHint().height())
@@ -345,7 +353,7 @@ class CompletionView(QTreeView):
         indexes = selected.indexes()
         if not indexes:
             return
-        data = self.model().data(indexes[0])
+        data = str(self.model().data(indexes[0]))
         self.selection_changed.emit(data)
 
     def resizeEvent(self, e):
@@ -365,9 +373,7 @@ class CompletionView(QTreeView):
                        modes=[usertypes.KeyMode.command], scope='window')
     def completion_item_del(self):
         """Delete the current completion item."""
-        if not self.currentIndex().isValid():
+        index = self.currentIndex()
+        if not index.isValid():
             raise cmdexc.CommandError("No item selected!")
-        try:
-            self.model().srcmodel.delete_cur_item(self)
-        except NotImplementedError:
-            raise cmdexc.CommandError("Cannot delete this item.")
+        self.model().delete_cur_item(index)
