@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2016 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -24,15 +24,14 @@ import base64
 import itertools
 import functools
 
-import jinja2
 from PyQt5.QtCore import pyqtSlot, QRect, QPoint, QTimer, Qt
 from PyQt5.QtWidgets import QWidget, QVBoxLayout, QApplication, QSizePolicy
 
 from qutebrowser.commands import runners, cmdutils
-from qutebrowser.config import config
-from qutebrowser.utils import message, log, usertypes, qtutils, objreg, utils
-from qutebrowser.mainwindow import tabbedbrowser, messageview, prompt
-from qutebrowser.mainwindow.statusbar import bar
+from qutebrowser.config import config, configfiles
+from qutebrowser.utils import (message, log, usertypes, qtutils, objreg, utils,
+                               jinja, debug)
+from qutebrowser.mainwindow import messageview, prompt
 from qutebrowser.completion import completionwidget, completer
 from qutebrowser.keyinput import modeman
 from qutebrowser.browser import (commands, downloadview, hints,
@@ -44,14 +43,18 @@ win_id_gen = itertools.count(0)
 
 
 def get_window(via_ipc, force_window=False, force_tab=False,
-               force_target=None):
+               force_target=None, no_raise=False):
     """Helper function for app.py to get a window id.
 
     Args:
         via_ipc: Whether the request was made via IPC.
         force_window: Whether to force opening in a window.
         force_tab: Whether to force opening in a tab.
-        force_target: Override the new-instance-open-target config
+        force_target: Override the new_instance_open_target config
+        no_raise: suppress target window raising
+
+    Return:
+        ID of a window that was used to open URL
     """
     if force_window and force_tab:
         raise ValueError("force_window and force_tab are mutually exclusive!")
@@ -60,7 +63,7 @@ def get_window(via_ipc, force_window=False, force_tab=False,
         # Initial main window
         return 0
 
-    open_target = config.get('general', 'new-instance-open-target')
+    open_target = config.val.new_instance_open_target
 
     # Apply any target overrides, ordered by precedence
     if force_target is not None:
@@ -72,33 +75,38 @@ def get_window(via_ipc, force_window=False, force_tab=False,
         open_target = 'tab-silent'
 
     window = None
-    raise_window = False
+    should_raise = False
 
     # Try to find the existing tab target if opening in a tab
     if open_target != 'window':
         window = get_target_window()
-        raise_window = open_target not in ['tab-silent', 'tab-bg-silent']
+        should_raise = open_target not in ['tab-silent', 'tab-bg-silent']
 
     # Otherwise, or if no window was found, create a new one
     if window is None:
-        window = MainWindow()
+        window = MainWindow(private=None)
         window.show()
-        raise_window = True
+        should_raise = True
 
-    if raise_window:
-        window.setWindowState(window.windowState() & ~Qt.WindowMinimized)
-        window.setWindowState(window.windowState() | Qt.WindowActive)
-        window.raise_()
-        window.activateWindow()
-        QApplication.instance().alert(window)
+    if should_raise and not no_raise:
+        raise_window(window)
 
     return window.win_id
 
 
+def raise_window(window):
+    """Raise the given MainWindow object."""
+    window.setWindowState(window.windowState() & ~Qt.WindowMinimized)
+    window.setWindowState(window.windowState() | Qt.WindowActive)
+    window.raise_()
+    window.activateWindow()
+    QApplication.instance().alert(window)
+
+
 def get_target_window():
     """Get the target window for new tabs, or None if none exist."""
     try:
-        win_mode = config.get('general', 'new-instance-open-target.window')
+        win_mode = config.val.new_instance_open_target_window
         if win_mode == 'last-focused':
             return objreg.last_focused_window()
         elif win_mode == 'first-opened':
@@ -123,20 +131,28 @@ class MainWindow(QWidget):
     Attributes:
         status: The StatusBar widget.
         tabbed_browser: The TabbedBrowser widget.
+        state_before_fullscreen: window state before activation of fullscreen.
         _downloadview: The DownloadView widget.
         _vbox: The main QVBoxLayout.
         _commandrunner: The main CommandRunner instance.
         _overlays: Widgets shown as overlay for the current webpage.
+        _private: Whether the window is in private browsing mode.
     """
 
-    def __init__(self, geometry=None, parent=None):
+    def __init__(self, *, private, geometry=None, parent=None):
         """Create a new main window.
 
         Args:
             geometry: The geometry to load, as a bytes-object (or None).
+            private: Whether the window is in private browsing mode.
             parent: The parent the window should get.
         """
         super().__init__(parent)
+        # Late import to avoid a circular dependency
+        # - browsertab -> hints -> webelem -> mainwindow -> bar -> browsertab
+        from qutebrowser.mainwindow import tabbedbrowser
+        from qutebrowser.mainwindow.statusbar import bar
+
         self.setAttribute(Qt.WA_DeleteOnClose)
         self._commandrunner = None
         self._overlays = []
@@ -161,7 +177,14 @@ class MainWindow(QWidget):
         self._init_downloadmanager()
         self._downloadview = downloadview.DownloadView(self.win_id)
 
-        self.tabbed_browser = tabbedbrowser.TabbedBrowser(self.win_id)
+        if config.val.content.private_browsing:
+            # This setting always trumps what's passed in.
+            private = True
+        else:
+            private = bool(private)
+        self._private = private
+        self.tabbed_browser = tabbedbrowser.TabbedBrowser(win_id=self.win_id,
+                                                          private=private)
         objreg.register('tabbed-browser', self.tabbed_browser, scope='window',
                         window=self.win_id)
         self._init_command_dispatcher()
@@ -169,7 +192,8 @@ class MainWindow(QWidget):
         # We need to set an explicit parent for StatusBar because it does some
         # show/hide magic immediately which would mean it'd show up as a
         # window.
-        self.status = bar.StatusBar(self.win_id, parent=self)
+        self.status = bar.StatusBar(win_id=self.win_id, private=private,
+                                    parent=self)
 
         self._add_widgets()
         self._downloadview.show()
@@ -184,8 +208,6 @@ class MainWindow(QWidget):
 
         self._keyhint = keyhintwidget.KeyHintView(self.win_id, self)
         self._add_overlay(self._keyhint, self._keyhint.update_geometry)
-        self._messageview = messageview.MessageView(parent=self)
-        self._add_overlay(self._messageview, self._messageview.update_geometry)
 
         self._prompt_container = prompt.PromptContainer(self.win_id, self)
         self._add_overlay(self._prompt_container,
@@ -195,6 +217,24 @@ class MainWindow(QWidget):
                         scope='window', window=self.win_id)
         self._prompt_container.hide()
 
+        self._messageview = messageview.MessageView(parent=self)
+        self._add_overlay(self._messageview, self._messageview.update_geometry)
+
+        self._init_geometry(geometry)
+        self._connect_signals()
+
+        # When we're here the statusbar might not even really exist yet, so
+        # resizing will fail. Therefore, we use singleShot QTimers to make sure
+        # we defer this until everything else is initialized.
+        QTimer.singleShot(0, self._connect_overlay_signals)
+        config.instance.changed.connect(self._on_config_changed)
+
+        objreg.get("app").new_window.emit(self)
+
+        self.state_before_fullscreen = self.windowState()
+
+    def _init_geometry(self, geometry):
+        """Initialize the window geometry or load it from disk."""
         if geometry is not None:
             self._load_geometry(geometry)
         elif self.win_id == 0:
@@ -204,16 +244,6 @@ class MainWindow(QWidget):
         log.init.debug("Initial main window geometry: {}".format(
             self.geometry()))
 
-        self._connect_signals()
-
-        # When we're here the statusbar might not even really exist yet, so
-        # resizing will fail. Therefore, we use singleShot QTimers to make sure
-        # we defer this until everything else is initialized.
-        QTimer.singleShot(0, self._connect_overlay_signals)
-        objreg.get('config').changed.connect(self.on_config_changed)
-
-        objreg.get("app").new_window.emit(self)
-
     def _add_overlay(self, widget, signal, *, centered=False, padding=0):
         self._overlays.append((widget, signal, centered, padding))
 
@@ -236,7 +266,7 @@ class MainWindow(QWidget):
             left = (self.width() - width) / 2 if centered else 0
 
         height_padding = 20
-        status_position = config.get('ui', 'status-position')
+        status_position = config.val.statusbar.position
         if status_position == 'bottom':
             if self.status.isVisible():
                 status_height = self.status.height()
@@ -290,7 +320,7 @@ class MainWindow(QWidget):
     def _init_completion(self):
         self._completion = completionwidget.CompletionView(self.win_id, self)
         cmd = objreg.get('status-command', scope='window', window=self.win_id)
-        completer_obj = completer.Completer(cmd, self.win_id, self._completion)
+        completer_obj = completer.Completer(cmd, self._completion)
         self._completion.selection_changed.connect(
             completer_obj.on_selection_changed)
         objreg.register('completion', self._completion, scope='window',
@@ -309,16 +339,14 @@ class MainWindow(QWidget):
     def __repr__(self):
         return utils.get_repr(self)
 
-    @pyqtSlot(str, str)
-    def on_config_changed(self, section, option):
+    @pyqtSlot(str)
+    def _on_config_changed(self, option):
         """Resize the completion if related config options changed."""
-        if section != 'ui':
-            return
-        if option == 'statusbar-padding':
+        if option == 'statusbar.padding':
             self._update_overlay_geometries()
-        elif option == 'downloads-position':
+        elif option == 'downloads.position':
             self._add_widgets()
-        elif option == 'status-position':
+        elif option == 'statusbar.position':
             self._add_widgets()
             self._update_overlay_geometries()
 
@@ -327,10 +355,9 @@ class MainWindow(QWidget):
         self._vbox.removeWidget(self.tabbed_browser)
         self._vbox.removeWidget(self._downloadview)
         self._vbox.removeWidget(self.status)
-        downloads_position = config.get('ui', 'downloads-position')
-        status_position = config.get('ui', 'status-position')
         widgets = [self.tabbed_browser]
 
+        downloads_position = config.val.downloads.position
         if downloads_position == 'top':
             widgets.insert(0, self._downloadview)
         elif downloads_position == 'bottom':
@@ -338,6 +365,7 @@ class MainWindow(QWidget):
         else:
             raise ValueError("Invalid position {}!".format(downloads_position))
 
+        status_position = config.val.statusbar.position
         if status_position == 'top':
             widgets.insert(0, self.status)
         elif status_position == 'bottom':
@@ -350,9 +378,8 @@ class MainWindow(QWidget):
 
     def _load_state_geometry(self):
         """Load the geometry from the state file."""
-        state_config = objreg.get('state-config')
         try:
-            data = state_config['geometry']['mainwindow']
+            data = configfiles.state['geometry']['mainwindow']
             geom = base64.b64decode(data, validate=True)
         except KeyError:
             # First start
@@ -365,10 +392,9 @@ class MainWindow(QWidget):
 
     def _save_geometry(self):
         """Save the window geometry to the state config."""
-        state_config = objreg.get('state-config')
         data = bytes(self.saveGeometry())
         geom = base64.b64encode(data).decode('ASCII')
-        state_config['geometry']['mainwindow'] = geom
+        configfiles.state['geometry']['mainwindow'] = geom
 
     def _load_geometry(self, geom):
         """Load geometry from a bytes object.
@@ -399,8 +425,6 @@ class MainWindow(QWidget):
 
     def _connect_signals(self):
         """Connect all mainwindow signals."""
-        key_config = objreg.get('key-config')
-
         status = self._get_object('statusbar')
         keyparsers = self._get_object('keyparsers')
         completion_obj = self._get_object('completion')
@@ -422,7 +446,8 @@ class MainWindow(QWidget):
         # commands
         keyparsers[usertypes.KeyMode.normal].keystring_updated.connect(
             status.keystring.setText)
-        cmd.got_cmd.connect(self._commandrunner.run_safely)
+        cmd.got_cmd[str].connect(self._commandrunner.run_safely)
+        cmd.got_cmd[str, int].connect(self._commandrunner.run_safely)
         cmd.returnPressed.connect(tabs.on_cmd_return_pressed)
 
         # key hint popup
@@ -430,37 +455,33 @@ class MainWindow(QWidget):
             parser.keystring_updated.connect(functools.partial(
                 self._keyhint.update_keyhint, mode.name))
 
-        # config
-        for obj in keyparsers.values():
-            key_config.changed.connect(obj.on_keyconfig_changed)
-
         # messages
         message.global_bridge.show_message.connect(
             self._messageview.show_message)
         message.global_bridge.flush()
+        message.global_bridge.clear_messages.connect(
+            self._messageview.clear_messages)
 
         message_bridge.s_set_text.connect(status.set_text)
         message_bridge.s_maybe_reset_text.connect(status.txt.maybe_reset_text)
 
         # statusbar
-        tabs.current_tab_changed.connect(status.prog.on_tab_changed)
+        tabs.current_tab_changed.connect(status.on_tab_changed)
+
         tabs.cur_progress.connect(status.prog.setValue)
         tabs.cur_load_finished.connect(status.prog.hide)
         tabs.cur_load_started.connect(status.prog.on_load_started)
 
-        tabs.current_tab_changed.connect(status.percentage.on_tab_changed)
         tabs.cur_scroll_perc_changed.connect(status.percentage.set_perc)
-
         tabs.tab_index_changed.connect(status.tabindex.on_tab_index_changed)
 
-        tabs.current_tab_changed.connect(status.url.on_tab_changed)
         tabs.cur_url_changed.connect(status.url.set_url)
+        tabs.cur_url_changed.connect(functools.partial(
+            status.backforward.on_tab_cur_url_changed, tabs=tabs))
         tabs.cur_link_hovered.connect(status.url.set_hover_url)
         tabs.cur_load_status_changed.connect(status.url.on_load_status_changed)
-        tabs.page_fullscreen_requested.connect(
-            self._on_page_fullscreen_requested)
-        tabs.page_fullscreen_requested.connect(
-            status.on_page_fullscreen_requested)
+        tabs.cur_fullscreen_requested.connect(self._on_fullscreen_requested)
+        tabs.cur_fullscreen_requested.connect(status.maybe_hide)
 
         # command input / completion
         mode_manager.left.connect(tabs.on_mode_left)
@@ -469,11 +490,14 @@ class MainWindow(QWidget):
         cmd.hide_completion.connect(completion_obj.hide)
 
     @pyqtSlot(bool)
-    def _on_page_fullscreen_requested(self, on):
+    def _on_fullscreen_requested(self, on):
         if on:
+            self.state_before_fullscreen = self.windowState()
             self.showFullScreen()
-        else:
-            self.showNormal()
+        elif self.isFullScreen():
+            self.setWindowState(self.state_before_fullscreen)
+        log.misc.debug('on: {}, state before fullscreen: {}'.format(
+            on, debug.qflags_key(Qt, self.state_before_fullscreen)))
 
     @cmdutils.register(instance='main-window', scope='window')
     @pyqtSlot()
@@ -524,24 +548,23 @@ class MainWindow(QWidget):
         if crashsignal.is_crashing:
             e.accept()
             return
-        confirm_quit = config.get('ui', 'confirm-quit')
         tab_count = self.tabbed_browser.count()
         download_model = objreg.get('download-model', scope='window',
                                     window=self.win_id)
         download_count = download_model.running_downloads()
         quit_texts = []
         # Ask if multiple-tabs are open
-        if 'multiple-tabs' in confirm_quit and tab_count > 1:
+        if 'multiple-tabs' in config.val.confirm_quit and tab_count > 1:
             quit_texts.append("{} {} open.".format(
                 tab_count, "tab is" if tab_count == 1 else "tabs are"))
         # Ask if multiple downloads running
-        if 'downloads' in confirm_quit and download_count > 0:
+        if 'downloads' in config.val.confirm_quit and download_count > 0:
             quit_texts.append("{} {} running.".format(
                 download_count,
                 "download is" if download_count == 1 else "downloads are"))
         # Process all quit messages that user must confirm
-        if quit_texts or 'always' in confirm_quit:
-            msg = jinja2.Template("""
+        if quit_texts or 'always' in config.val.confirm_quit:
+            msg = jinja.environment.from_string("""
                 <ul>
                 {% for text in quit_texts %}
                    <li>{{text}}</li>
