@@ -39,6 +39,7 @@ except ImportError:
     colorama = None
 
 _log_inited = False
+_args = None
 
 COLORS = ['black', 'red', 'green', 'yellow', 'blue', 'purple', 'cyan', 'white']
 COLOR_ESCAPES = {color: '\033[{}m'.format(i)
@@ -94,7 +95,7 @@ LOGGER_NAMES = [
     'commands', 'signals', 'downloads',
     'js', 'qt', 'rfc6266', 'ipc', 'shlexer',
     'save', 'message', 'config', 'sessions',
-    'webelem', 'prompt', 'network'
+    'webelem', 'prompt', 'network', 'sql'
 ]
 
 
@@ -141,6 +142,7 @@ sessions = logging.getLogger('sessions')
 webelem = logging.getLogger('webelem')
 prompt = logging.getLogger('prompt')
 network = logging.getLogger('network')
+sql = logging.getLogger('sql')
 
 
 ram_handler = None
@@ -188,8 +190,9 @@ def init_log(args):
     logging.captureWarnings(True)
     _init_py_warnings()
     QtCore.qInstallMessageHandler(qt_message_handler)
-    global _log_inited
+    global _log_inited, _args
     _log_inited = True
+    _args = args
 
 
 def _init_py_warnings():
@@ -344,7 +347,7 @@ def qt_message_handler(msg_type, context, msg):
     try:
         qt_to_logging[QtCore.QtInfoMsg] = logging.INFO
     except AttributeError:
-        # Qt < 5.5
+        # While we don't support Qt < 5.5 anymore, logging still needs to work
         pass
 
     # Change levels of some well-known messages to debug so they don't get
@@ -408,6 +411,8 @@ def qt_message_handler(msg_type, context, msg):
         # https://codereview.qt-project.org/176831
         "QObject::disconnect: Unexpected null parameter",
     ]
+    # not using utils.is_mac here, because we can't be sure we can successfully
+    # import the utils module here.
     if sys.platform == 'darwin':
         suppressed_msgs += [
             'libpng warning: iCCP: known incorrect sRGB profile',
@@ -439,7 +444,11 @@ def qt_message_handler(msg_type, context, msg):
         msg += ("\n\nOn Archlinux, this should fix the problem:\n"
                 "    pacman -S libxkbcommon-x11")
         faulthandler.disable()
-    stack = ''.join(traceback.format_stack())
+
+    if _args.debug:
+        stack = ''.join(traceback.format_stack())
+    else:
+        stack = None
     record = qt.makeRecord(name, level, context.file, context.line, msg, None,
                            None, func, sinfo=stack)
     qt.handle(record)
